@@ -47,6 +47,7 @@ typedef struct {
 
 	TpChannel      *publish;
 	TpChannel      *subscribe;
+	TpChannel      *stored;
 	GHashTable     *members; /* handle -> EmpathyContact */
 	GHashTable     *pendings; /* handle -> EmpathyContact */
 	GHashTable     *groups; /* group name -> TpChannel */
@@ -122,6 +123,75 @@ tp_contact_list_group_invalidated_cb (TpChannel *channel,
 	g_hash_table_remove (priv->groups, group_name);
 }
 
+static void
+contacts_added_to_group (EmpathyTpContactList *list,
+			 TpChannel *channel,
+			 GArray *added)
+{
+	EmpathyTpContactListPriv *priv = GET_PRIV (list);
+	const gchar *group_name;
+	guint i;
+
+	group_name = tp_channel_get_identifier (channel);
+
+	for (i = 0; i < added->len; i++) {
+		EmpathyContact *contact;
+		TpHandle handle;
+
+		handle = g_array_index (added, TpHandle, i);
+		contact = g_hash_table_lookup (priv->members,
+					       GUINT_TO_POINTER (handle));
+		if (contact == NULL) {
+			continue;
+		}
+
+		DEBUG ("Contact %s (%d) added to group %s",
+			empathy_contact_get_id (contact), handle, group_name);
+		g_signal_emit_by_name (list, "groups-changed", contact,
+				       group_name,
+				       TRUE);
+	}
+}
+
+static void
+tp_contact_list_group_members_changed_cb (TpChannel     *channel,
+					  gchar         *message,
+					  GArray        *added,
+					  GArray        *removed,
+					  GArray        *local_pending,
+					  GArray        *remote_pending,
+					  guint          actor,
+					  guint          reason,
+					  EmpathyTpContactList *list)
+{
+	EmpathyTpContactListPriv  *priv = GET_PRIV (list);
+	const gchar *group_name;
+	guint i;
+
+	contacts_added_to_group (list, channel, added);
+
+	group_name = tp_channel_get_identifier (channel);
+
+	for (i = 0; i < removed->len; i++) {
+		EmpathyContact *contact;
+		TpHandle handle;
+
+		handle = g_array_index (removed, TpHandle, i);
+		contact = g_hash_table_lookup (priv->members,
+					       GUINT_TO_POINTER (handle));
+		if (contact == NULL) {
+			continue;
+		}
+
+		DEBUG ("Contact %s (%d) removed from group %s",
+			empathy_contact_get_id (contact), handle, group_name);
+
+		g_signal_emit_by_name (list, "groups-changed", contact,
+				       group_name,
+				       FALSE);
+	}
+}
+
 static void
 tp_contact_list_group_ready_cb (TpChannel *channel,
 				const GError *error,
@@ -130,6 +200,8 @@ tp_contact_list_group_ready_cb (TpChannel *channel,
 	EmpathyTpContactListPriv *priv = GET_PRIV (list);
 	TpChannel *old_group;
 	const gchar *group_name;
+	const TpIntSet *members;
+	GArray *arr;
 
 	if (error) {
 		DEBUG ("Error: %s", error->message);
@@ -158,6 +230,10 @@ tp_contact_list_group_ready_cb (TpChannel *channel,
 	g_hash_table_insert (priv->groups, (gpointer) group_name, channel);
 	DEBUG ("Group %s added", group_name);
 
+	g_signal_connect (channel, "group-members-changed",
+			  G_CALLBACK (tp_contact_list_group_members_changed_cb),
+			  list);
+
 	g_signal_connect (channel, "invalidated",
 			  G_CALLBACK (tp_contact_list_group_invalidated_cb),
 			  list);
@@ -173,61 +249,13 @@ tp_contact_list_group_ready_cb (TpChannel *channel,
 			g_hash_table_remove (priv->add_to_group, group_name);
 		}
 	}
-}
 
-static void
-tp_contact_list_group_members_changed_cb (TpChannel     *channel,
-					  gchar         *message,
-					  GArray        *added,
-					  GArray        *removed,
-					  GArray        *local_pending,
-					  GArray        *remote_pending,
-					  guint          actor,
-					  guint          reason,
-					  EmpathyTpContactList *list)
-{
-	EmpathyTpContactListPriv  *priv = GET_PRIV (list);
-	const gchar *group_name;
-	guint i;
-
-	group_name = tp_channel_get_identifier (channel);
-
-	for (i = 0; i < added->len; i++) {
-		EmpathyContact *contact;
-		TpHandle handle;
-
-		handle = g_array_index (added, TpHandle, i);
-		contact = g_hash_table_lookup (priv->members,
-					       GUINT_TO_POINTER (handle));
-		if (contact == NULL) {
-			continue;
-		}
-
-		DEBUG ("Contact %s (%d) added to group %s",
-			empathy_contact_get_id (contact), handle, group_name);
-		g_signal_emit_by_name (list, "groups-changed", contact,
-				       group_name,
-				       TRUE);
-	}
-
-	for (i = 0; i < removed->len; i++) {
-		EmpathyContact *contact;
-		TpHandle handle;
-
-		handle = g_array_index (removed, TpHandle, i);
-		contact = g_hash_table_lookup (priv->members,
-					       GUINT_TO_POINTER (handle));
-		if (contact == NULL) {
-			continue;
-		}
-
-		DEBUG ("Contact %s (%d) removed from group %s",
-			empathy_contact_get_id (contact), handle, group_name);
-
-		g_signal_emit_by_name (list, "groups-changed", contact,
-				       group_name,
-				       FALSE);
-	}
+	/* Get initial members of the group */
+	members = tp_channel_group_get_members (channel);
+	g_assert (members != NULL);
+	arr = tp_intset_to_array (members);
+	contacts_added_to_group (list, channel, arr);
+	g_array_free (arr, TRUE);
 }
 
 static void
@@ -250,11 +278,6 @@ tp_contact_list_group_add_channel (EmpathyTpContactList *list,
 				  object_path, channel_type,
 				  handle_type, handle, NULL);
 
-	/* TpChannel emits initial set of members just before being ready */
-	g_signal_connect (channel, "group-members-changed",
-			  G_CALLBACK (tp_contact_list_group_members_changed_cb),
-			  list);
-
 	/* Give the ref to the callback */
 	tp_channel_call_when_ready (channel,
 				    tp_contact_list_group_ready_cb,
@@ -485,32 +508,6 @@ tp_contact_list_publish_group_members_changed_cb (TpChannel     *channel,
 	}
 }
 
-static void
-tp_contact_list_publish_request_channel_cb (TpConnection *connection,
-					    const gchar  *object_path,
-					    const GError *error,
-					    gpointer      user_data,
-					    GObject      *list)
-{
-	EmpathyTpContactListPriv *priv = GET_PRIV (list);
-
-	if (error) {
-		DEBUG ("Error: %s", error->message);
-		return;
-	}
-
-	priv->publish = tp_channel_new (connection, object_path,
-					TP_IFACE_CHANNEL_TYPE_CONTACT_LIST,
-					TP_HANDLE_TYPE_LIST,
-					GPOINTER_TO_UINT (user_data),
-					NULL);
-
-	/* TpChannel emits initial set of members just before being ready */
-	g_signal_connect (priv->publish, "group-members-changed",
-			  G_CALLBACK (tp_contact_list_publish_group_members_changed_cb),
-			  list);
-}
-
 static void
 tp_contact_list_get_alias_flags_cb (TpConnection *connection,
 				    guint         flags,
@@ -569,31 +566,6 @@ tp_contact_list_get_requestablechannelclasses_cb (TpProxy      *connection,
 	}
 }
 
-static void
-tp_contact_list_publish_request_handle_cb (TpConnection *connection,
-					   const GArray *handles,
-					   const GError *error,
-					   gpointer      user_data,
-					   GObject      *list)
-{
-	TpHandle handle;
-
-	if (error) {
-		DEBUG ("Error: %s", error->message);
-		return;
-	}
-
-	handle = g_array_index (handles, TpHandle, 0);
-	tp_cli_connection_call_request_channel (connection, -1,
-						TP_IFACE_CHANNEL_TYPE_CONTACT_LIST,
-						TP_HANDLE_TYPE_LIST,
-						handle,
-						TRUE,
-						tp_contact_list_publish_request_channel_cb,
-						GUINT_TO_POINTER (handle), NULL,
-						list);
-}
-
 static void
 tp_contact_list_subscribe_group_members_changed_cb (TpChannel     *channel,
 						    gchar         *message,
@@ -632,57 +604,6 @@ tp_contact_list_subscribe_group_members_changed_cb (TpChannel     *channel,
 	}
 }
 
-static void
-tp_contact_list_subscribe_request_channel_cb (TpConnection *connection,
-					      const gchar  *object_path,
-					      const GError *error,
-					      gpointer      user_data,
-					      GObject      *list)
-{
-	EmpathyTpContactListPriv *priv = GET_PRIV (list);
-
-	if (error) {
-		DEBUG ("Error: %s", error->message);
-		return;
-	}
-
-	priv->subscribe = tp_channel_new (connection, object_path,
-					  TP_IFACE_CHANNEL_TYPE_CONTACT_LIST,
-					  TP_HANDLE_TYPE_LIST,
-					  GPOINTER_TO_UINT (user_data),
-					  NULL);
-
-	/* TpChannel emits initial set of members just before being ready */
-	g_signal_connect (priv->subscribe, "group-members-changed",
-			  G_CALLBACK (tp_contact_list_subscribe_group_members_changed_cb),
-			  list);
-}
-
-static void
-tp_contact_list_subscribe_request_handle_cb (TpConnection *connection,
-					     const GArray *handles,
-					     const GError *error,
-					     gpointer      user_data,
-					     GObject      *list)
-{
-	TpHandle handle;
-
-	if (error) {
-		DEBUG ("Error: %s", error->message);
-		return;
-	}
-
-	handle = g_array_index (handles, TpHandle, 0);
-	tp_cli_connection_call_request_channel (connection, -1,
-						TP_IFACE_CHANNEL_TYPE_CONTACT_LIST,
-						TP_HANDLE_TYPE_LIST,
-						handle,
-						TRUE,
-						tp_contact_list_subscribe_request_channel_cb,
-						GUINT_TO_POINTER (handle), NULL,
-						list);
-}
-
 static void
 tp_contact_list_new_channel_cb (TpConnection *proxy,
 				const gchar  *object_path,
@@ -750,6 +671,9 @@ tp_contact_list_finalize (GObject *object)
 	if (priv->publish) {
 		g_object_unref (priv->publish);
 	}
+	if (priv->stored) {
+		g_object_unref (priv->stored);
+	}
 
 	if (priv->connection) {
 		g_object_unref (priv->connection);
@@ -773,13 +697,93 @@ tp_contact_list_finalize (GObject *object)
 	G_OBJECT_CLASS (empathy_tp_contact_list_parent_class)->finalize (object);
 }
 
+static void
+list_ensure_channel_cb (TpConnection *conn,
+			gboolean yours,
+			const gchar *path,
+			GHashTable *properties,
+			const GError *error,
+			gpointer user_data,
+			GObject *weak_object)
+{
+	EmpathyTpContactList *list = user_data;
+	EmpathyTpContactListPriv *priv = GET_PRIV (list);
+	const gchar *id;
+	TpChannel *channel;
+
+	if (error != NULL) {
+		DEBUG ("failed: %s\n", error->message);
+		return;
+	}
+
+	/* We requested that channel by providing TargetID property, so it's
+	 * guaranteed that tp_channel_get_identifier will return it. */
+	channel = tp_channel_new_from_properties (conn, path, properties, NULL);
+	id = tp_channel_get_identifier (channel);
+
+	/* TpChannel emits initial set of members just before being ready */
+	if (!tp_strdiff (id, "stored")) {
+		priv->stored = channel;
+	} else if (!tp_strdiff (id, "publish")) {
+		priv->publish = channel;
+		g_signal_connect (priv->publish, "group-members-changed",
+				  G_CALLBACK (tp_contact_list_publish_group_members_changed_cb),
+				  list);
+	} else if (!tp_strdiff (id, "subscribe")) {
+		priv->subscribe = channel;
+		g_signal_connect (priv->subscribe, "group-members-changed",
+				  G_CALLBACK (tp_contact_list_subscribe_group_members_changed_cb),
+				  list);
+	} else {
+		g_warn_if_reached ();
+		g_object_unref (channel);
+	}
+}
+
+static void
+conn_ready_cb (TpConnection *connection,
+	       const GError *error,
+	       gpointer data)
+{
+	EmpathyTpContactList *list = data;
+	EmpathyTpContactListPriv *priv = GET_PRIV (list);
+	GHashTable *request;
+
+	if (error != NULL) {
+		DEBUG ("failed: %s", error->message);
+		goto out;
+	}
+
+	request = tp_asv_new (
+		TP_IFACE_CHANNEL ".ChannelType", G_TYPE_STRING, TP_IFACE_CHANNEL_TYPE_CONTACT_LIST,
+		TP_IFACE_CHANNEL ".TargetHandleType", G_TYPE_UINT, TP_HANDLE_TYPE_LIST,
+		NULL);
+
+	/* Request the 'stored' list. */
+	tp_asv_set_static_string (request, TP_IFACE_CHANNEL ".TargetID", "stored");
+	tp_cli_connection_interface_requests_call_ensure_channel (priv->connection,
+		-1, request, list_ensure_channel_cb, list, NULL, G_OBJECT (list));
+
+	/* Request the 'publish' list. */
+	tp_asv_set_static_string (request, TP_IFACE_CHANNEL ".TargetID", "publish");
+	tp_cli_connection_interface_requests_call_ensure_channel (priv->connection,
+		-1, request, list_ensure_channel_cb, list, NULL, G_OBJECT (list));
+
+	/* Request the 'subscribe' list. */
+	tp_asv_set_static_string (request, TP_IFACE_CHANNEL ".TargetID", "subscribe");
+	tp_cli_connection_interface_requests_call_ensure_channel (priv->connection,
+		-1, request, list_ensure_channel_cb, list, NULL, G_OBJECT (list));
+
+	g_hash_table_unref (request);
+out:
+	g_object_unref (list);
+}
+
 static void
 tp_contact_list_constructed (GObject *list)
 {
-
 	EmpathyTpContactListPriv *priv = GET_PRIV (list);
 	gchar                    *protocol_name = NULL;
-	const gchar              *names[] = {NULL, NULL};
 
 	priv->factory = empathy_tp_contact_factory_dup_singleton (priv->connection);
 
@@ -809,22 +813,8 @@ tp_contact_list_constructed (GObject *list)
 		priv->flags |= EMPATHY_CONTACT_LIST_CAN_GROUP;
 	}
 
-	names[0] = "publish";
-	tp_cli_connection_call_request_handles (priv->connection,
-						-1,
-						TP_HANDLE_TYPE_LIST,
-						names,
-						tp_contact_list_publish_request_handle_cb,
-						NULL, NULL,
-						G_OBJECT (list));
-	names[0] = "subscribe";
-	tp_cli_connection_call_request_handles (priv->connection,
-						-1,
-						TP_HANDLE_TYPE_LIST,
-						names,
-						tp_contact_list_subscribe_request_handle_cb,
-						NULL, NULL,
-						G_OBJECT (list));
+	tp_connection_call_when_ready (priv->connection, conn_ready_cb,
+		g_object_ref (list));
 
 	tp_cli_connection_call_list_channels (priv->connection, -1,
 					      tp_contact_list_list_channels_cb,
@@ -994,6 +984,14 @@ tp_contact_list_remove (EmpathyContactList *list,
 	GArray handles = {(gchar *) &handle, 1};
 
 	handle = empathy_contact_get_handle (contact);
+
+	/* FIXME: this is racy if tp_contact_list_remove is called before the
+	 * 'stored' list has been retrieved. */
+	if (priv->stored != NULL) {
+		tp_cli_channel_interface_group_call_remove_members (priv->stored,
+			-1, &handles, message, NULL, NULL, NULL, NULL);
+	}
+
 	if (priv->subscribe) {
 		tp_cli_channel_interface_group_call_remove_members (priv->subscribe,
 			-1, &handles, message, NULL, NULL, NULL, NULL);
