@@ -300,6 +300,8 @@ account_assistant_account_enabled_cb (GObject *source,
       salut_created = TRUE;
     }
 
+  empathy_connect_new_account (account, priv->account_mgr);
+
   current_idx = gtk_assistant_get_current_page (GTK_ASSISTANT (self));
   if (current_idx == PAGE_SALUT && !salut_created)
     /* We are on the Salut page and aren't creating the salut account so don't
@@ -322,22 +324,9 @@ account_assistant_apply_account_cb (GObject *source,
   EmpathyAccountAssistantPriv *priv = GET_PRIV (self);
   EmpathyAccountSettings *settings = EMPATHY_ACCOUNT_SETTINGS (source);
   TpAccount *account;
-  gchar *display_name;
 
   empathy_account_settings_apply_finish (settings, result, &error);
 
-  if (empathy_account_settings_get_display_name (settings) == NULL)
-    {
-      /* set default display name */
-      display_name = empathy_account_widget_get_default_display_name (
-          priv->current_widget_object);
-
-      empathy_account_settings_set_display_name_async (settings,
-          display_name, NULL, NULL);
-
-      g_free (display_name);
-    }
-
   priv->is_creating = FALSE;
 
   if (error != NULL)
@@ -359,12 +348,22 @@ account_assistant_apply_account_and_finish (EmpathyAccountAssistant *self,
     EmpathyAccountSettings *settings)
 {
   EmpathyAccountAssistantPriv *priv = GET_PRIV (self);
+  gchar *display_name;
 
   if (settings == NULL)
     return;
 
   priv->is_creating = TRUE;
 
+  /* set default display name */
+  display_name = empathy_account_widget_get_default_display_name (
+      priv->current_widget_object);
+
+  empathy_account_settings_set_display_name_async (settings,
+      display_name, NULL, NULL);
+
+  g_free (display_name);
+
   empathy_account_settings_apply_async (settings,
       account_assistant_apply_account_cb, self);
 }
